﻿Type=StaticCode
Version=2.45
@EndOfDesignText@
'StateManager v1.12
'Code module
Public Sub Process_Globals
	Private states As Map
	Private listPosition As Int
	Private statesFileName, settingsFileName As String
	Public settings As Map
	Public SettingsDeleteResult As Boolean = False
	Public StateDeleteResult As Boolean = False
	
	statesFileName = "state.dat"
	settingsFileName = "settings.properties"
End Sub

'Gets the setting value associated with the given key.
'Returns the DefaultValue parameter if the key was not found.
Public Sub GetSetting2(Key As String, DefaultValue As String)
	If settings.IsInitialized = False Then
		'load the stored settings
		If File.Exists(File.DirInternal, settingsFileName) Then 
			settings = File.ReadMap(File.DirInternal, settingsFileName)
		Else
			Return DefaultValue
		End If
	End If
	
	Return settings.GetDefault(Key.ToLowerCase, DefaultValue)
	
End Sub

'Gets the setting value associated with the given key.
'Returns an empty string if the key was not found.
Public Sub GetSetting(Key As String)
	Return GetSetting2(Key, "")
End Sub

Public Sub SetSetting(Key As String, Value As String)
	If settings.IsInitialized = False Then
		'load the stored settings
		If File.Exists(File.DirInternal, settingsFileName) Then 
			settings = File.ReadMap(File.DirInternal, settingsFileName)
		Else
			settings.Initialize
		End If
	End If
	
	settings.Put(Key.ToLowerCase, Value)
	Log ("Wrote " & Key.ToLowerCase & " (" & Value & ") to SETTINGS file")
End Sub

'Stored the settings in a file
Public Sub SaveSettings
	If settings.IsInitialized Then
		File.WriteMap(File.DirInternal, settingsFileName, settings)
		Log ("SETTINGS saved")
	End If
End Sub

'Resets the stored state data for this activity.
Public Sub ResetState(ActivityName As String)
	loadStateFile
	
	If states.IsInitialized Then 
		states.Remove(ActivityName.ToLowerCase)
		writeStateToFile
		StateDeleteResult = False
		SettingsDeleteResult = False
	End If
End Sub

'Saves the activity state. ActivityName - Used for handling multiple activities.
Public Sub SaveState(Activity As Activity, ActivityName As String)
	If states.IsInitialized = False Then states.Initialize
	Dim ActivityList As List
	
	ActivityList.Initialize
	ActivityList.Add(DateTime.Now)
	
	For i = 0 To Activity.NumberOfViews - 1
		innerSaveState(Activity.GetView(i), ActivityList)
	Next
	
	states.Put(ActivityName.ToLowerCase, ActivityList)
	writeStateToFile
End Sub

Private Sub writeStateToFile
	Dim raf As RandomAccessFile
	
	raf.Initialize(File.DirInternal, statesFileName, False)
	raf.WriteObject(states, True, raf.CurrentPosition)
	raf.Close
	
	Log ("STATE saved")
End Sub

Private Sub innerSaveState(TheView As View, list1 As List)
	Dim data() As Object
	
	If TheView Is EditText Then
		Dim edit As EditText
		
		edit = TheView
		data = Array As Object(edit.Text, edit.SelectionStart)
	Else If TheView Is Spinner Then
		Dim spinner1 As Spinner
		
		spinner1 = TheView
		data = Array As Object(spinner1.SelectedIndex)
	Else If TheView Is CheckBox Then
		Dim check As CheckBox
		
		check = TheView
		data = Array As Object(check.Checked)
	Else If TheView Is RadioButton Then
		Dim radio As RadioButton
		
		radio = TheView
		data = Array As Object(radio.Checked)	
	Else If TheView Is ToggleButton Then
		Dim toggle As ToggleButton
		
		toggle = TheView
		data = Array As Object(toggle.Checked)
	Else If TheView Is SeekBar Then
		Dim seek As SeekBar
		
		seek = TheView
		data = Array As Object(seek.Value)
	Else If TheView Is TabHost Then
		Dim th As TabHost
		Dim data() As Object
		Dim R As Reflector
		Dim tabParentPanel As Panel
		
		th = TheView
		data = Array As Object(th.CurrentTab)
		For i = 0 To th.TabCount - 1
			th.CurrentTab = i
		Next
		list1.Add(data)
		
		R.Target = th
		
		tabParentPanel = R.RunMethod("getTabContentView")
		For i = 0 To tabParentPanel.NumberOfViews - 1
			innerSaveState(tabParentPanel.GetView(i), list1)
		Next
	Else If TheView Is ScrollView Then
		Dim sv As ScrollView
		Dim data() As Object
		
		sv = TheView
		data = Array As Object(sv.ScrollPosition)
		list1.Add(data)
		
		innerSaveState(sv.Panel, list1)
	Else If TheView Is Panel Then
		Dim panel1 As Panel
		
		panel1 = TheView
		For i = 0 To panel1.NumberOfViews - 1
			innerSaveState(panel1.GetView(i), list1)
		Next
	End If
	
	If data.Length > 0 Then list1.Add(data)
End Sub

Private Sub innerRestoreState(TheView As View, list1 As List)
	Dim data() As Object
	
	If TheView Is EditText Then
		Dim edit As EditText
		
		edit = TheView
		data = getNextItem(list1)
		edit.Text = data(0)
		edit.SelectionStart = data(1)
	Else If TheView Is Spinner Then
		Dim spinner1 As Spinner
		
		spinner1 = TheView
		data = getNextItem(list1)
		spinner1.SelectedIndex = data(0)
	Else If TheView Is CheckBox Then
		Dim check As CheckBox
		
		check = TheView
		data = getNextItem(list1)
		check.Checked = data(0)
	Else If TheView Is RadioButton Then
		Dim radio As RadioButton
		
		radio = TheView
		data = getNextItem(list1)
		radio.Checked = data(0)
	Else If TheView Is ToggleButton Then
		Dim toggle As ToggleButton
		
		toggle = TheView
		data = getNextItem(list1)
		toggle.Checked = data(0)
	Else If TheView Is SeekBar Then
		Dim seek As SeekBar
		
		seek = TheView
		data = getNextItem(list1)
		seek.Value = data(0)
	Else If TheView Is TabHost Then
		Dim th As TabHost
		Dim R As Reflector
		Dim tabParentPanel As Panel
		
		th = TheView
		data = getNextItem(list1)
		For i = 0 To th.TabCount - 1
			th.CurrentTab = i
		Next
		th.CurrentTab = data(0)
		
		R.Target = th
		
		tabParentPanel = R.RunMethod("getTabContentView")
		For i = 0 To tabParentPanel.NumberOfViews - 1
			innerRestoreState(tabParentPanel.GetView(i), list1)
		Next
	Else If TheView Is ScrollView Then
		Dim sv As ScrollView
		
		sv = TheView
		data = getNextItem(list1)
		sv.ScrollPosition = data(0)
		DoEvents
		sv.ScrollPosition = data(0)
		innerRestoreState(sv.Panel, list1)
	Else If TheView Is Panel Then
		Dim panel1 As Panel
		
		panel1 = TheView
		For i = 0 To panel1.NumberOfViews - 1
			innerRestoreState(panel1.GetView(i), list1)
		Next
	End If
End Sub

Private Sub getNextItem(list1 As List) As Object()
	listPosition = listPosition + 1
	
	Log ("SETTING item " & listPosition & " loaded")
	
	Return list1.Get(listPosition)
End Sub

'Loads the stored state (if such is available)
'ActivityName - Should match the value use in SaveState
'ValidPeriodInMinutes - The validity period of this state measured in minutes. Pass 0 for an unlimited period.
'Returns true if the state was loaded
Public Sub RestoreState(Activity As Activity, ActivityName As String, ValidPeriodInMinutes As Int) As Boolean
	Try
		Dim list1 As List
		Dim time As Long
		
		loadStateFile
		If states.IsInitialized = False Then
			Return False
		End If
		
		list1 = states.Get(ActivityName.ToLowerCase)
		If list1.IsInitialized = False Then Return
		
		time = list1.Get(0)
		If ValidPeriodInMinutes > 0 AND time + ValidPeriodInMinutes * DateTime.TicksPerMinute < DateTime.Now Then
			Return False
		End If
		
		listPosition = 0
		
		For i = 0 To Activity.NumberOfViews - 1
			innerRestoreState(Activity.GetView(i), list1)
		Next
		Return True
	Catch
		Log("Error loading state.")
		Log(LastException.Message)
		Return False
	End Try
End Sub

Private Sub loadStateFile
	'only load the state if it is not already available in memory.
	If states.IsInitialized Then Return
	If File.Exists(File.DirInternal, statesFileName) Then
		Dim raf As RandomAccessFile
		
		raf.Initialize(File.DirInternal, statesFileName, False)
		states = raf.ReadObject(0)
		raf.Close
		
		Log ("STATE loaded")
	End If
End Sub

'Deletes the stored state (if such is available)
'Returns true if the state was deleted
Public Sub DeleteState As Boolean
	If File.Exists(File.DirInternal, statesFileName) Then
		Try
			File.Delete(File.DirInternal, statesFileName)
			
			Log ("State file deleted")
			
			StateDeleteResult = True
			Return True
		Catch
			Log ("Error deleting State file")
		End Try
	Else
		Log ("State file not found")
	End If
End Sub

'Deletes the stored settings (if such is available)
'Returns true if the settings were deleted
Public Sub DeleteSettings As Boolean
	If File.Exists(File.DirInternal, settingsFileName) Then
		Try
			File.Delete(File.DirInternal, settingsFileName)
			
			Log ("Settings file deleted")
			
			SettingsDeleteResult = True
			Return True
		Catch
			Log ("Error deleting Settings file")
		End Try
	Else
		Log ("Settings file not found")
	End If
End Sub

'Deletes the stored state and settings (if such is available)
'Returns true if both were deleted
'Check StateDeleteResult and SettingsDeleteResult for further info if FALSE is returned
Public Sub DeleteStateAndSettings As Boolean
	If DeleteSettings AND DeleteState Then
		Return True
	End If
End Sub
